#!/usr/bin/env python
# vim: et sta sts ai ts=4 sw=4:
import os
import socket
import sys

from charmhelpers.core import hookenv, host
from charmhelpers.core.hookenv import log
from charmhelpers.fetch import apt_install, apt_update, add_source
from charmhelpers.contrib.cloudfoundry.upstart_helper import (
    install_upstart_scripts
)
import yaml

ROUTER_PACKAGES = ['cfgorouter']
ROUTER_CONFIG_FILE = '/var/lib/cloudfoundry/config/gorouter.yml'
hooks = hookenv.Hooks()


def emit_router_config(context):
    with open('templates/gorouter.yml') as fh:
        router_cfg = yaml.safe_load(fh.read())
        
    nats = router_cfg['nats'][0]
    nats['host'] = context['nats_address']
    nats['port'] = context['nats_port']
    nats['user'] = context['nats_user']
    nats['pass'] = context['nats_password']
    host.write_file(
        ROUTER_CONFIG_FILE, yaml.safe_dump(router_cfg),
        perms=0400)


@hooks.hook()
def install():
    config = hookenv.config()
    add_source(config['source'])
    apt_update(fatal=True)
    apt_install(packages=ROUTER_PACKAGES, fatal=True)
    dirs = ["/var/vcap/sys/log/gorouter",
            "/var/vcap/sys/run/gorouter",
            "/var/lib/cloudfoundry/config",
            "/var/vcap/sys/log/gorouter"]
    for dir in dirs:
        host.mkdir(dir, perms=0775)
    install_upstart_scripts()


@hooks.hook()
def start():
    hookenv.open_port(80)
    hookenv.open_port(443)


def default_domain():
    domain = hookenv.unit_get('public-address')
    domain = "%s.xip.io" % (socket.gethostbyname(domain))
    return domain


@hooks.hook("config-changed")
def config_changed():
    config = hookenv.config()
    domain = config.get('domain')
    if not domain:
        domain = default_domain()

    for rel_id in hookenv.relation_ids('router'):
        hookenv.relation_set(rel_id, {'domain': domain})


@hooks.hook()
def stop():
    if host.service_running('gorouter'):
        host.service_stop('gorouter')
    hookenv.close_port(80)
    hookenv.close_port(443)


@hooks.hook('nats-relation-changed')
def nats_relation_changed():
    emit_router_config(hookenv.relation_get())
    host.service_restart('gorouter')


@hooks.hook('nats-relation-broken')
def nats_relation_broken():
    stop()


@hooks.hook('router-relation-changed')
def router_relation_changed():
    domain = hookenv.config().get('domain')
    if not domain:
        domain = default_domain()
    hookenv.relation_set(None, {'domain': domain})


@hooks.hook('router-relation-joined')
def router_relation_joined():
    pass


if __name__ == '__main__':
    hook_name = os.path.basename(sys.argv[0])
    log("Running {} hook".format(hook_name))
    if hookenv.relation_id():
        log("Relation {} with {}".format(
            hookenv.relation_id(), hookenv.remote_unit()))
    hooks.execute(sys.argv)
